@@ -83,7 +83,7 @@ summary.dispRity <- function(object, ..., quantiles = c(50, 95), cent.tend = med
         #Set to default (see below)
         digits <- "default"
     } else {
-        check.class(digits, "numeric")
+        check.class(digits, c("numeric", "integer"))
     }
 
     #DATA
@@ -149,15 +149,8 @@ summary.dispRity <- function(object, ..., quantiles = c(50, 95), cent.tend = med
                                         do.call(rbind, lapply(simulation_results, function(X) rbind(X$quantiles))))
             colnames(simulation_results)[1] <- as.character(match_call$cent.tend)
 
-            ## Get inherited subsets (if exist)
-            if(!is.null(data$subsets)) {
-                subset_names <- rev(data$subsets)
-            } else {
-                subset_names <- rev(data$simulation.data$fix$subsets)
-            }
-
             ## Output table
-            output_table <- cbind("subsets" = subset_names,
+            output_table <- cbind("subsets" = rev(data$simulation.data$fix$subsets),
                                   "n" = data$simulation.data$fix$sample_size,
                                   "var" = unname(data$simulation.data$fix$variance),
                                   simulation_results)
@@ -165,6 +158,52 @@ summary.dispRity <- function(object, ..., quantiles = c(50, 95), cent.tend = med
             return(output_table)
         }
 
+
+        if(is(data, "test.metric")) {
+
+            ## Summary table
+            get.cent.tends <- function(results) {
+                combined <- aggregate(disparity~reduction, data = results, FUN = function(x, cent.tend) cent.tend(x, na.rm = TRUE), cent.tend, na.action = na.pass)
+                return(combined[order(as.numeric(combined$reduction)),2])
+            }
+            all_results <- round(do.call(rbind, lapply(data$results, get.cent.tends)), digits = ifelse(digits == "default", 2, digits))
+            colnames(all_results) <- paste0(unique(data$results[[1]][,"reduction"]), "%")
+
+            ## Adding the model results
+            if(!is.null(data$models)) {
+                get.model.summary <- function(model) {
+                    ## Try to get the model parameters
+                    try_slope <- try.get.from.model(model, "Estimate")
+                    try_p_val <- try.get.from.model(model, "Pr\\(")
+                    try_r_squ <- try.get.from.model(model, "r.squared")
+                    if(!is.null(try_r_squ) && any(names(try_r_squ) == "adj.r.squared")) {
+                        try_r_squ <- try_r_squ$adj.r.squared
+                        is_adjusted <- TRUE
+                    } else {
+                        is_adjusted <- FALSE
+                    }
+
+                    ## Make a results vector
+                    results <- c(
+                        "slope" = ifelse(any(!is.null(try_slope)) && any(!is.na(try_slope)), try_slope["reduction"], NA),
+                        "p_val" = ifelse(any(!is.null(try_p_val)) && any(!is.na(try_p_val)), try_p_val["reduction"], NA),
+                        "r_squ" = ifelse(any(!is.null(try_r_squ)) && any(!is.na(try_r_squ)), try_r_squ, NA))
+                            
+                    names(results) <- c("slope", "p_value", ifelse(is_adjusted, "R^2(adj)", "R^2"))
+
+                    return(results)
+                }
+
+                ## Get the model summaries
+                model_summaries <- do.call(rbind, lapply(data$models, get.model.summary))
+
+                ## Combine both
+                all_results <- cbind(all_results, model_summaries)
+            }
+
+            return(all_results)
+        }
+
         ## No dual class summary available
         stop.call("", paste0("No specific summary for combined class \"dispRity\" and \"", class(data)[2], "\"."))
     } 
@@ -187,7 +226,7 @@ summary.dispRity <- function(object, ..., quantiles = c(50, 95), cent.tend = med
             elements[[null_elem]] <- nrow(data$subsets[[null_elem]]$elements)
         }
     }
-    
+
     ## Get the names of the subsets
     names <- names(data$subsets)
     if(is.null(names)) {
@@ -198,29 +237,58 @@ summary.dispRity <- function(object, ..., quantiles = c(50, 95), cent.tend = med
     disparity_values <- lapply(data$disparity, lapply.observed)
     names(disparity_values) <- NULL
 
-    ## Initialise the results
-    summary_results <- data.frame(row.names = NULL, "subsets" = rep(names, unlist(lapply(elements, length))), "n" = unlist(elements))
+
+    ## Change the names and subsets if between groups
+    if(data$call$disparity$metrics$between.groups) {
+        ## Groups names
+        names <- names(data$disparity)
+
+        ## Remove warning (desired: matching rows with different numbers of elements!)
+        options(warn = -1)
+        ## Get the table of elements for each pair of groups
+        elements_matrix <-lapply(as.list(names),  function(name, elements)
+                                do.call(cbind, lapply(as.list(strsplit(name, split = ":")[[1]]), function(name, elements) return(unname(elements[[match(name, names(elements))]])), elements)),
+                                elements = elements)
+        options(warn = 0)
+
+        ## Initialise the results
+        summary_results <- data.frame(row.names = NULL,
+                                    "subsets" = rep(names, unlist(lapply(elements_matrix, nrow))),
+                                    "n_1" = do.call(rbind,elements_matrix)[,1],
+                                    "n_2" = do.call(rbind,elements_matrix)[,2])
+    } else {
+        ## Initialise the results        
+        summary_results <- data.frame(row.names = NULL, "subsets" = rep(names, unlist(lapply(elements, length))), "n" = unlist(elements))
+    }
+
+    ## Set the observed colume
+    if(data$call$disparity$metrics$between.groups) {
+        obs_col <- 4
+        elements <- lapply(elements_matrix, function(x) seq(1:nrow(x)))
+    } else {
+        obs_col <- 3
+    }
 
     ## Add the observed values
     if(is_distribution) {
         summary_results <- cbind(summary_results, as.vector(unlist(mapply(mapply.observed, lapply(disparity_values, cent.tend, ...), elements))), row.names = NULL)
-        names(summary_results)[3] <- paste("obs", as.expression(match_call$cent.tend), sep = ".")
+        names(summary_results)[obs_col] <- paste("obs", as.expression(match_call$cent.tend), sep = ".")
     } else {
         summary_results <- cbind(summary_results, as.vector(unlist(mapply(mapply.observed, disparity_values, elements))), row.names = NULL)
-        names(summary_results)[3] <- "obs"
+        names(summary_results)[obs_col] <- "obs"
     }
 
     if(!is.null(data$call$bootstrap)) {
         ## Calculate the central tendencies and the quantiles
         summary_results <- cbind(summary_results, matrix(unlist(lapply(data$disparity, lapply.summary, cent.tend, quantiles, ...)), byrow = TRUE, ncol = (1+length(quantiles)*2)))
         ## Adding the labels
-        names(summary_results)[4:length(summary_results)] <- c(paste("bs", as.expression(match_call$cent.tend), sep = "."), names(quantile(rnorm(5), probs = CI.converter(quantiles))))
+        names(summary_results)[(obs_col+1):length(summary_results)] <- c(paste("bs", as.expression(match_call$cent.tend), sep = "."), names(quantile(rnorm(5), probs = CI.converter(quantiles))))
     } else {
         if(is_distribution) {
             ## Calculate the quantiles
             summary_results <- cbind(summary_results, matrix(unlist(lapply(data$disparity, lapply, get.summary, quantiles = quantiles)), byrow = TRUE, ncol = (length(quantiles)*2)))
             ## Adding the labels
-            names(summary_results)[4:length(summary_results)] <- c(names(quantile(rnorm(5), probs = CI.converter(quantiles))))
+            names(summary_results)[(obs_col+1):length(summary_results)] <- c(names(quantile(rnorm(5), probs = CI.converter(quantiles))))
         }
     }
 
@@ -228,9 +296,15 @@ summary.dispRity <- function(object, ..., quantiles = c(50, 95), cent.tend = med
     summary_results <- digits.fun(summary_results, digits)
 
     ## If any elements is equal to one, check if not NA
-    if(any(summary_results$n == 1)) {
+    if(data$call$disparity$metrics$between.groups) {
+        col_n <- c(2,3)
+    } else {
+        col_n <- 2
+    }
+
+    if(any(summary_results[, col_n] == 1)) {
         ## Select the values to check
-        to_check <- which(summary_results$n == 1)
+        to_check <- which(summary_results[, col_n] == 1)
         ## Get their replacement values
 
         check.elements.NA <- function(row, summary_results, data) {
@@ -240,7 +314,7 @@ summary.dispRity <- function(object, ..., quantiles = c(50, 95), cent.tend = med
 
         replace_vals <- sapply(to_check, check.elements.NA, summary_results, data)
         ## Replace them in the results
-        summary_results[to_check, 2] <- replace_vals
+        summary_results[to_check, col_n] <- replace_vals
     }
 
     #----------------------
